
#import <appkit/appkit.h>

#import "GraphRight_API.h"
#import "GraphRightClient.h"

#define N 10
#define M 6

@implementation GraphRightClient

- appDidInit:sender
{
    BOOL didLaunch =[[Application workspace] launchApplication:"GraphRight.app"];
    return self;
}

- connectToServer:sender
{
    const char *serverHostname = [hostName stringValue];

    grServer = [NXConnection connectToName:"GraphRightServer"
	                            onHost:serverHostname];

    if(!grServer)
    {
	[messageFromServerField setStringValue:"No GraphRight Server Found"];
	return self;
    }
    else
    {
	[messageFromServerField setStringValue:"GraphRight Server Found"];
    }
    
    
    return self;
}

- sendData:sender
{
    id notebook;

    if(!grServer)
    {
	[messageFromServerField setStringValue:"Not Connected toGraphRight Server"];
	return self;
    }
    /* we are connected, so now check for the notebook we want */
    notebook = [grServer GR_getOpenNotebook:"GR Client Notebook"];

    if(!notebook)
    {
	/* we could not find a notebook called "GR Client Notebook" on
	   the server, we need to create a new notebook and save it
	   in our home directory
	 */
	notebook = [grServer GR_createNewNotebook];
	if([notebook GR_setPath:"~/GR Client Notebook"] != NO_ERROR)
	{
	    /* for some reason we can't create the notebook, or the
	       path given was bad... were dead...
	     */
	    [messageFromServerField setStringValue:"Could not create Notebook : GR Client Notebook"];
	    return self;
	}
    }

    /* now we know we have a valid notebook to work with */
    [self sendChunckOfDataTo:notebook];
    
    return self;
}

- sendChunckOfDataTo:notebook
{
    int i,j;
    int n = N;
    int m = M;
    GRCell cell;
    NXTypedStream *ts;
    NXStream *stream = NXOpenMemory(NULL, 0, NX_WRITEONLY);
    char *buffer;
    int len, maxLen;
    id data;
    
    ts = NXOpenTypedStream(stream,  NX_WRITEONLY);
    i = m * n;
    NXWriteType(ts, "i", &i); /* Write the number of cells we are sending */
    for(i=0;i<n;i++)
	for(j=0;j<m;j++)
	{
	    cell.r = i;
	    cell.c = j;
	    cell.isValue = YES;
	    cell.val = random()/10000000;
	    cell.string = NXCopyStringBuffer("");
/* This MUST be done, it seems that there is a bug in Distributed Objects when   
 * sending a NULL string. OD's don't check for NULL and just try to do a 
 * strlen(string) on the NULL, this crashes the client
 */
	    NXWriteType(ts, "{iicf*}", &cell);
	}
    
    NXCloseTypedStream(ts);
    /* encapsule the data */
    NXGetMemoryBuffer(stream, &buffer, &len, &maxLen);
    data = [[NXData alloc] initWithData:buffer size:len dealloc:YES];
    /* send to client, NXData objects are automatically sent 'bycopy' */

    [notebook GR_setCells:data]; /* currentDoc is a Proxy to a DataDocument */
    [data free];
    
    return NO_ERROR;
}

- createGraph:sender
{
    id notebook;
    GRAddress upperLeft, lowerRight;
    
    if(!grServer)
    {
	[messageFromServerField setStringValue:"Not Connected to GraphRight Server"];
	return self;
    }
    /* we are connected, so now check for the notebook we want */
    notebook = [grServer GR_getOpenNotebook:"GR Client Notebook"];
    
    if(!notebook)
    {
	[messageFromServerField setStringValue:"Could not find GR Client Notebook"];
	return self;
    }

    /* Ok we have found it, now select the data we know is there and graph it */
    
    upperLeft.r = 0;
    upperLeft.c = 0;
    lowerRight.r = N;
    lowerRight.c = M;
    
    [notebook GR_selectRange:upperLeft bottom:lowerRight];

    /* Now lets create the Graph */

    if([notebook GR_createGraphType:GR_STACKEDCOLUMN_CHART] == NULL)
	[messageFromServerField setStringValue:
	 "Error: can't create graph: invalid data selected"];
    else
	[messageFromServerField setStringValue:"Successfully created graph"];
    
    return self;
}

@end
